/*                                                                          
  Ϳ
   Ints.h  (versin 0.1)                                                  
                                                                          
   Librera con algunas funciones ASM usando interrupciones.              
                                                                          
                                                                          
   Cdigo y texto:  [NoP] de Compiler SoftWare.                     
  1997 C.S.
*/

/*  Funciones implementadas:  */

/* INTERRUPCIONES SOFTWARE */

void SetVideoMode( char );               /* inicializar modos de video */
char GetVideoMode( void );               /* leer el modo de video actual */
void SetBorder( char );                  /* Cambiar el color del borde */
void WaitVRetrace( void );               /* espera al retrazo vertical */


/*                                                                          
  Ŀ
   SetVideoMode( char modo );                                             
                                                                          
   Inicializa el modo de video especificado en el parmetro <modo>, sien- 
   do ste un nmero entre 0 y 0x13 (Ver modos de video en la revista).   
   Utiliza el servicio 0 (Set VMode) de la int 10h.                       
  
*/

void SetVideoMode( char modo )
{
asm {
        xor ah, ah                       /* AH = 0 (Servicio SetVideoMode) */
        mov al, [modo]                   /* AL = Modo de video */
        int 10h                          /* llamada a la interrupcin 10h */
    }
}


/*                                                                          
  Ŀ
   modo = GetVideoMode( void );                                           
                                                                          
   Devuelve el modo de video actual. Utiliza int 10h, servicio 0Fh.       
  
*/

char GetVideoMode( void )
{
char modo;
asm {
        mov ah, 0x0F                     /* AH = 0Fh (GetVideoMode) */
        int 10h                          /* llamada a la interrupcin 10h */
        mov [modo], al                   /* AL = Modo de video */
    }
 return( modo );                         /* devolver el valor leido */
}


/*                                                                          
  Ŀ
   SetBorder( char color );                                               
                                                                          
   Modifica el color del borde mediante el servicio 1001h de la int 10h.  
   No es muy til pero es otro ejemplo de uso de interrupciones.          
  
*/

void SetBorder( char color )
{
asm {
        mov ax, 0x1001                   /* AX = 01001h (SetBorderColour) */
        mov bh, [color]                  /* BH = Color del borde */
        int 10h                          /* llamada a la interrupcin 10h */
    }
}


/*                                                                          
  Ŀ
   void Int_PutPixel( x, y, color );                                      
                                                                          
   Dibuja el pixel (x,y) con el color <color> en la pgina 0. Funciona    
   para cualquier modo de video grfico estandar inicializado con el      
   servicio 0 (SetVideoMode) de la int 10h. Hay que recordar que las in-  
   terrupciones son demasiado lentas para su uso en buenos programas.     
  
*/

void Int_PutPixel( int x, int y, char color )
{
asm {
        mov ah, 0x0C                     /* AH = 0x0C (WritePixel) */
        mov al, [color]                  /* AL = Color del pixel */
        mov cx, [x]
        mov dx, [y]                      /* (CX,DX) = (x,y) */
        xor bh, bh                       /* pgina 0 */
        int 10h                          /* llamada a la interrupcin 10h */
    }
}

/*                                                                          
  Ŀ
   void Int_GetPixel( x, y, color );                                      
                                                                          
   Lee el color del pixel (x,y) usando el servicio 0Dh de la int 10h.     
  
*/

char Int_GetPixel( int x, int y )
{
char color;

asm {
        mov ah, 0x0D                     /* AH = 0x0D (ReadPixel) */
        mov cx, [x]
        mov dx, [y]                      /* (CX,DX) = (x,y) */
        xor bh, bh                       /* pgina 0 */
        int 10h                          /* llamada a la interrupcin 10h */
        mov [color], al                  /* AL = Color del pixel */
    }

   return(color);                        /* devolver el valor leido */
}

/*------------------------------------------------------------------------*/
void WaitVRetrace( void )
{
 asm mov dx, 0x3da
ret1:;
 asm {
        in al, dx
        test al, 8
        jnz ret1
     }
ret2:;
 asm {
        in al, dx
        test al, 8
        jz ret2
     }
}

