/*                                                                          
  Ϳ
   Mode13h.h  (versin 0.2)                                               
                                                                          
   Librera grfica especfica para modo 13h (320x200x256).               
   Contiene acceso directo a VideoRAM y proporciona las primeras rutinas  
   (sern optimizadas conforme se avance) de dibujo en pantalla.          
                                                                          
   Para incluirla en cualquier programa C basta con usar la orden         
   #include "mode13h.h"                 al principio del mismo.           
                                                                          
   Usa instrucciones 286, compilar con la opcin -1 del compilador.       
   Requiere DOS.H para la funcin pokeb() de PutPixelC();                 
                                                                          
   Cdigo y texto:  [NoP] de Compiler SoftWare.                     
  1997 C.S.
*/

/*  Funciones implementadas:  */

/* FUNCIONES DE MODO 13h */

void ClearScreen( char );                /* Limpiar la pantalla en la VRAM */
void PutPixelMul( int, int, char );        /* PutPixel a la VRAM (*320) */
void PutPixelC( int, int, char );        /* PutPixel a la VRAM (*320) */
void PutPixel( int, int, char );         /* PutPixel a la VRAM (shl) */
char GetPixel( int, int );               /* GetPixel con acceso a la VRAM */
void DrawHLine( int, int, int, char );   /* Lineas horizontales con STOSB */
void DrawVLine( int, int, int, char );   /* Lineas verticales en la VRAM */
void DrawBox( int, int, int, int, char );         /* Cajas usando 4 lneas */
void FillBox( int, int, int, int, char );     /* Cajas rellenas por HLines */


/*                                                                          
  Ŀ
   ClearScreen( char color );                                             
                                                                          
   Borra la pantalla en el modo 13h rellenando el segmento de la VideoRAM 
   (segmento 0A000h) con el color especificado.                           
   Utiliza rdenes de cadena assembler (rep stosb) para mayor velocidad.  
                                                                          
   Pseudocdigo:                                                          
     ES:DI -> A000:0000h                                                  
     AL = Color                                                           
     CX = 320*200 = 64.000 bytes a escribir.                              
     Repetir CX veces                                                     
        ES:[DI] = AL                                                      
        DI = DI + 1                                                       
  
*/

void ClearScreen( char color )
{
asm {
        mov ax, 0xA000
        mov es, ax
        xor di, di                        /* ES:DI -> A000:0000h */
        mov al, [color]                   /* AL = Color */
        mov cx, 64000                     /* Repetir CX veces (320x200)*/
        rep stosb                         /* ES:[DI++] = AL */
    }
}                       


/*                                                                          
  Ŀ
   PutPixelC( int x, int y, char color );                                 
                                                                          
   Pone el pixel del coordenadas (x,y) con el color <color>, escribiendo  
   en la VideoRAM dentro del offset (320*y)+x. Usa pokeb() de DOS.H.      
                                                                          
   Pseudocdigo:                                                          
    Offset = (320*y) + x                                                  
    A000:[Offset] = Color                                                 
  
*/
void PutPixelC( int x, int y, char color )
{
unsigned int offset_pixel;

 offset_pixel = (320*y) + x;              /* offset del punto en la VRAM */
 pokeb( 0xA000, offset_pixel, color );
}

/*                                                                          
  Ŀ
   PutPixelMul( int x, int y, char color );                               
                                                                          
   Pone el pixel del coordenadas (x,y) con el color <color>, escribiendo  
   en la VideoRAM dentro del offset (320*y)+x. Ver el artculo para ms   
   explicaciones.                                                         
                                                                          
   Pseudocdigo:                                                          
    Offset = (320*y) + x                                                  
    A000:[Offset] = Color                                                 
  
*/

void PutPixelMul( int x, int y, char color )
{
unsigned int offset_pixel;

 offset_pixel = (320*y) + x;              /* offset del punto en la VRAM */
asm {
        mov ax, 0xA000
        mov es, ax
        mov di, [offset_pixel]            /* ES:DI -> A000:Offset */
        mov al, [color]                   /* AL = Color */
        mov es:[di], al                   /* ES:[DI] = AL */
    }
}


/*                                                                          
  Ŀ
   PutPixel( int x, int y, char color );                                  
                                                                          
   Pone el pixel del coordenadas (x,y) con el color <color>, escribiendo  
   en la VideoRAM dentro del offset (320*y)+x. Utiliza desplazamientos    
   para multiplicar con mayor velocidad.                                  
                                                                          
   Pseudocdigo:                                                          
    Offset = (y<<8) + (y<<6) + x;                                         
    A000:[Offset] = Color                                                 
  
*/

void PutPixel( int x, int y, char color )
{
unsigned int offset_pixel;

 offset_pixel = (y<<8) + (y<<6) + x;      /* offset del punto en la VRAM */

asm {
        mov ax, 0xA000
        mov es, ax
        mov di, [offset_pixel]            /* ES:DI -> A000:Offset */
        mov al, [color]                   /* AL = Color */
        mov es:[di], al                   /* ES:[DI] = AL */
    }
}


/*                                                                          
  Ŀ
   color = GetPixel( int x, int y );                                      
                                                                          
   Devuelve el color del pixel (x,y) leyendo su valor de la memoria de    
   video. Utiliza desplazamientos para multiplicar la y por 320.          
                                                                          
   Pseudocdigo:                                                          
    Offset = (y<<8) + (y<<6) + x;                                         
    Color = A000:[Offset] = Color                                         
  
*/

char GetPixel( int x, int y )
{
unsigned int offset_pixel;

 offset_pixel = (y<<8) + (y<<6) + x;      /* offset del punto en la VRAM */

asm {
        mov ax, 0xA000
        mov es, ax
        mov di, [offset_pixel]            /* ES:DI -> A000:Offset */
        mov al, es:[di]                   /* AL = ES:[DI] */
    }

 return( _AL );                         /* devolver color del pixel */
}


/*                                                                          
  Ŀ
   DrawHLine( int x, int y, int ancho, char color );                      
                                                                          
   Dibuja una lnea horizontal desde (x,y) hasta (x+ancho,y) (de <ancho>  
   pixels de anchura) usando el color <color>.                            
                                                                          
   Pseudocdigo:                                                          
    OffsetXY = (y<<8) + (y<<6) + x;                                       
    CX = Ancho                                                            
    Repetir CX veces                                                      
      A000:[DI++] = Color                                                 
                                                                          
   Como stosb escribe CX bytes, si hacemos, por ejemplo, CX = 10, el REP  
   STOSB escribir 10 bytes consecutivos en memoria con el valor AL, y    
   como es VideoRAM, en pantalla aparecern 10 pixels consecutivos de ese 
   color: hemos trazado una linea horizontal aprovechando el assembler,   
   en vez de haber hecho un bucle como este (que sera MUCHO ms lento)   
                                                                          
    for( f=0; f<ancho; f++)                                               
      PutPixel( x+f, y, Color );                                          
  
*/

void DrawHLine( int x, int y, int ancho, char color )
{
unsigned int offset_pixel;

 offset_pixel = (y<<8) + (y<<6) + x;      /* offset del punto en la VRAM */

asm {
        mov ax, 0xA000
        mov es, ax
        mov di, [offset_pixel]            /* ES:DI -> A000:Offset */
        mov al, [color]                   /* AL = Color */
        mov cx, [ancho]                   /* Repetir CX veces */
        rep stosb                         /* ES:[DI++] = AL */
    }
}


/*                                                                          
  Ŀ
   DrawVLine( int x, int y, int alto, char color );                       
                                                                          
   Dibuja una lnea vertical desde (x,y) hasta (x,y+alto) (de <alto>      
   pixels de altura) usando el color <color>.                             
                                                                          
   Pseudocdigo:                                                          
    OffsetXY = (y<<8) + (y<<6) + x;                                       
    CX = Ancho                                                            
    Repetir CX veces                                                      
      A000:[DI] = Color                                                   
      DI = DI + 320            (siguiente lnea de la pantalla)           
                                                                          
   Incrementamos DI en 320 porque, igual que incrementarlo en 1 es el     
   pixel de al lado, si la pantalla tiene 320 pixels y nos los saltamos,  
   estamos dibujando en el pixel de abajo. Para entender esto veamos el   
   siguiente ejemplo:                                                     
                                                                          
                   Pantalla                       VideoRAM                
                                                                          
                 12                                              
                 3                                              
                 4     =     1234   
                                                               
                                                               
                                                                          
    Como puede verse en las 2 figuras, el punto 3 est a <anchopantalla>  
    puntos del punto 4. Si nuestra pantalla es de 320 bytes de ancho,     
    si incrementamos DI en 320 estamos en el punto de abajo.              
  
*/

void DrawVLine( int x, int y, int alto, char color )
{
unsigned int offset_pixel;

 offset_pixel = (y<<8) + (y<<6) + x;      /* offset del punto en la VRAM */

asm {
        mov ax, 0xA000
        mov es, ax
        mov di, [offset_pixel]            /* ES:DI -> A000:Offset */
        mov al, [color]                   /* AL = Color */
        mov cx, [alto]                    /* Repetir CX veces */
     }

BucleLinea:;
asm {
        mov es:[di], al                   /* ES:[DI] = Color */
        add di, 320                       /* Pixel de abajo (DI+=320) */
        loop BucleLinea                   /* repetir hasta que CX sea 0 */
    }
}


/*                                                                          
  Ŀ
   DrawBox( int x1, int y1, int ancho, int alto );                        
                                                                          
   Dibuja una lnea horizontal desde (x,y) hasta (x+ancho,y+alto) usando  
   el color <color>. Lo nico que hace esta funcin es dibujar un rectn- 
   gulo dibujando las 4 lneas que lo encierran (caja vaca). Las 4 lneas
   que dibuja son como las de este recuadro de texto: las dos de arriba y 
   las de los laterales, usando DrawHLine y DrawVLine.                    
                                                                          
          (x,y) 133333333332 (x+ancho,y)                                  
                1          2                 1 = DrawVLine(x,y,alto);     
                1          2                 2 = DrawVLine(x+an,y,alto);  
                1          2                 3 = DrawHLine(x,y,ancho);    
    (x,y+alto)  144444444442 (x+an,y+al)     4 = DrawHLine(x,y+alto,an);  
                                                                          
  
*/

void DrawBox( int x, int y, int ancho, int alto, char color )         
{
  
  DrawVLine(x, y, alto, color );
  DrawVLine(x+ancho, y, alto, color );          /* 2 lneas verticales */
  DrawHLine(x, y, ancho+1, color);
  DrawHLine(x, y+alto, ancho+1, color);           /* 2 lneas horizontales */

}


/*                                                                          
  Ŀ
   FillBox( int x1, int y1, int ancho, int alto );                        
                                                                          
   Rellena una caja desde (x,y) hasta (x+ancho,y+alto) usando             
   el color <color>. Lo nico que hace esta funcin es dibujar una caja   
   dibujando las alto lneas horizontales que la definen.                 
                                                                          
          (x,y) 111111111111 (x+ancho,y)                                  
                222222222222                                              
                333333333333           =    Dibujar <alto> lineas horiz.  
                444444444444                                              
    (x,y+alto)  555555555555 (x+an,y+al)                                  
                                                                          
  
*/

void FillBox( int x, int y, int ancho, int alto, char color )         
{
 int bucle;

 for( bucle=0; bucle<alto; bucle++ )
   DrawHLine( x, y+bucle, ancho, color );

}

/*--------------------------------------------------------------------------
 [END OF FILE]                                                   [NoP]
 --------------------------------------------------------------------------*/
